<?php


if(class_exists("AshWP"))
	return;

abstract class AshWPVersion{

	/***return version number string version.update.patch */
	abstract public function Version();

}

class AshWP{

	private static $instance = null;
	private $versions;

	

	private $slug;
	private $version;
	private $cacheKey;
	private $cacheAllowed;
	
	
	private function __construct(){
		$this->versions = array();

		$this->slug = plugin_basename( __DIR__ );
		$this->version = '0.0.0';
		$this->cacheKey = 'ash-custom-upd';
		$this->cacheAllowed = false;
		
		add_filter( 'plugins_api', array( $this, 'Info' ), 20, 3 );
		add_filter( 'site_transient_update_plugins', array( $this, 'Update' ) );
		add_action( 'upgrader_process_complete', array( $this, 'Purge' ), 10, 2 );

		/*thanks https://wordpress.stackexchange.com/questions/314701/remove-view-details-option-from-plugin-listing */
		add_filter('plugin_row_meta',function ($plugin_meta, $plugin_file, $plugin_data, $status) {
			$pluginSlug = explode("/", $plugin_file)[0];
			$path = $_SERVER["REQUEST_SCHEME"] . "://" . $_SERVER["HTTP_HOST"] . "/wp-admin/" . (is_multisite()? "network/" : "");

			if($pluginSlug  == $this->slug)
				$plugin_meta[2] = "<a href=\"{$path}plugin-install.php?tab=plugin-information&plugin={$pluginSlug}&;TB_iframe=true\" class=\"thickbox open-plugin-details-modal\">View details</a>";

			return $plugin_meta;
		},10,4);

		add_action( 'admin_enqueue_scripts', function($page)
		{
			if($page == "plugin-install.php" && isset($_GET["plugin"]) && $_GET["plugin"] == dirname(plugin_basename( __FILE__ )))
				wp_enqueue_style( "ash-wp-style", plugin_dir_url( __FILE__ ) . "ash-wp-style.css", array(), null);

		});


	}

	public static function Instance()
	{
		if (self::$instance == null)
		{
			self::$instance = new AshWP();
		}
		return self::$instance;
	}

	public function RegisterVersion($version)
	{
		if(!is_subclass_of($version, "AshWpVersion", false))
		{
			throw new ErrorException("Version must inherit from AshWpVersion, for example 'class NewVersion extends AshWpVersion{}'", 0);
		}

		$versionNum = $version->Version();
		if(!is_string($versionNum) || !preg_match("/^\d+\.\d+\.\d+$/", $versionNum))
		{
			throw new ErrorException("Version must be a string containg three numbers seperated by dots, release.update.patch, for example \"1.0.0\". Can be set by overloading Version() and returning \"1.0.0\"", 0);
		}

		$versionNums = explode(".", $versionNum);
		$position = 0;

		for($j=0; $j<count($this->versions); $j++)
		{
			$checkVersion = $this->versions[$j];
			$checkVersionNums = explode(".", $checkVersion->Version());
			for($i=0; $i<count($versionNums); $i++)
			{
				if(intval($versionNums[$i]) > intval($checkVersionNums[$i]))
				{
					$position = $j;
					break 2;
				}
			}
		}
		if(count($this->versions) <= $position)
		{
			$this->versions[] = $version;
		}
		else
		{
			array_splice($this->versions, $position, 0, [$version]);
		}
		
		return true;
	}

	

	public function __call( $method, $args )
	{
		if(!count($this->versions))
			throw new ErrorException("No version registerd, use AshWP::Instance()->RegisterVersion(new SpecialVersion())", 0);

		for($i=0; $i<count($this->versions); $i++)
		{
			$classObj = $this->versions[$i];
			if(method_exists($classObj, $method))
			{
				if($i != 0)
					trigger_error("depricated method, $method, you may be using an outdated AshWP Library.", E_USER_NOTICE);
				
				
				return call_user_func_array(array($classObj, $method), $args);
			}
		}

		throw new ErrorException("Method, $method, not found.", 0);
	}
	
	Private function Request(){

		$remote = get_transient( $this->cacheKey );

		if( false === $remote || ! $this->cacheAllowed ) {

			$remote = wp_remote_get(
				'https://amcgowan.co.uk/public/files/ash-wp/info.json',
				array(
					'timeout' => 10,
					'headers' => array(
						'Accept' => 'application/json'
					)
				)
			);

			if(
				is_wp_error( $remote )
				|| 200 !== wp_remote_retrieve_response_code( $remote )
				|| empty( wp_remote_retrieve_body( $remote ) )
			) {
				return false;
			}

			set_transient( $this->cacheKey, $remote, DAY_IN_SECONDS );

		}

		$remote = json_decode( wp_remote_retrieve_body( $remote ) );

		return $remote;

	}

	public function Info($res, $action, $args )
	{

			// print_r( $action );
			// print_r( $args );

			// do nothing if you're not getting plugin information right now
			if( 'plugin_information' !== $action ) {
				return $res;
			}

			// do nothing if it is not our plugin
			if( $this->slug !== $args->slug ) {
				return $res;
			}

			// get updates
			$remote = $this->Request();

			if( ! $remote ) {
				return $res;
			}

			$res = new stdClass();

			$res->name = $remote->name;
			$res->slug = $remote->slug;
			$res->version = $remote->version;
			$res->tested = $remote->tested;
			$res->requires = $remote->requires;
			$res->author = $remote->author;
			//$res->author_profile = $remote->author_profile;
			$res->download_link = $remote->download_url;
			$res->trunk = $remote->download_url;
			$res->requires_php = $remote->requires_php;
			$res->last_updated = $remote->last_updated;

			$res->sections = (array)$remote->sections;

			if( ! empty( $remote->banners ) ) {

				$res->banners = (array)$remote->banners;
			}

			return $res;
	}

	public function Update($transient)
	{

		if ( empty($transient->checked ) ) {
			return $transient;
		}

		$remote = $this->request();


		if(
			$remote
			&& version_compare( $this->version, $remote->version, '<' )
			&& version_compare( $remote->requires, get_bloginfo( 'version' ), '<' )
			&& version_compare( $remote->requires_php, PHP_VERSION, '<' )
		) 
		{
			$res = new \stdClass();
			$res->slug = $this->slug;
			$res->plugin = "ash-wp/ash-wp.php";
			$res->new_version = $remote->version;
			$res->tested = $remote->tested;
			$res->package = $remote->download_url;

			$transient->response[ $res->plugin ] = $res;

		}

		return $transient;
	}

	public function Purge()
	{

		if (  $this->cacheAllowed && 'update' === $options['action'] && 'plugin' === $options[ 'type' ] ) {
			// just clean the cache when new plugin version is installed
			delete_transient( $this->cacheKey );
		}
	}


	
}