<?php namespace AshWp\Standalone;

if(defined("AshWPConfigPage"))
	return;
else
	define("AshWPConfigPage", true);

/* TODO: OOP */

add_action("muplugins_loaded", function(){

	$filters = [
		"disableIncludes"=>"ModifyDefaultIncludes",
		"disableHeadData"=>"ModifyDefaultHeadTags",
		"disableFeatures"=>"ModifyDefaultFeatures",
	];

	$filterKeys = array_keys($filters);

	for($i=0; $i< count($filterKeys); $i++)
	{
		$key = $filterKeys[$i];
		$filter = $filters[$key];
		
		add_filter($filter, function($includes) use($key){
			$setting = get_option("ash-wp-settings", array());
			$setting = empty($setting) ? array() : $setting;
			$value = array_key_exists($key, $setting) ? $setting[$key] : null ;
			
			if($value == null)
				return $includes;

			//dont molest my arrays with your random pointers php
			$icopy = $includes;
			$vcopy = $value;
			sort($icopy);
			sort($vcopy);
			
			if($icopy == $vcopy)
				return $includes;

			return $value;
		});
	}

	$setting = get_option("ash-wp-settings", array());
	$setting = empty($setting) ? array() : $setting;
	$tagOptions = array_key_exists("tagOptions", $setting) ? $setting["tagOptions"] : null ;
	$adminBar = array_key_exists("adminBar", $setting) ? $setting["adminBar"] : null ;
	$pageData = array_key_exists("pageData", $setting) ? $setting["pageData"] : null ;


	if($tagOptions===false)
		do_action( "DisableTagOptions");

	if($adminBar===false)
	{
		add_filter("ModifyDefaultIncludes", function($includes){
			$includes[] = "admin-bar"; 
			return $includes;
		});
	}
	
	if($pageData===true)
		do_action( "ConsolePageData");

	//do_action( "WriteEnqued" );

	



});


add_action( "admin_menu", function(){

	$settignsPageHook = add_options_page( "Ashley's WordPress Library", "Ashley's WP Library", 'manage_options', 'ash-wp', function (){
	   ?>
	   <div class="wrap">
			<h1><?=esc_html_e( 'Ashleys Wordpress Library Config Page', 'ash-wp' ); ?></h1>
			<form method="POST" action="options.php">
				<?php settings_fields( "ash-wp" ); ?>
				<?php do_settings_sections_properly( "ash-wp" ); ?>
				<?php submit_button(); ?>
			</form>
		<div>
		<?php
	} );
	
	add_action("admin_enqueue_scripts", function($hook) use($settignsPageHook){
		if($hook !== $settignsPageHook)
			return;
		wp_enqueue_style( "ash-wp-style", plugin_dir_url( __FILE__ ) . "ash-wp-style.css", array(), null);
	});

} );


function do_settings_sections_properly($page)
{	
    global $wp_settings_sections, $wp_settings_fields;
 
    if ( ! isset( $wp_settings_sections[ $page ] ) ) {
        return;
    }

	foreach ( (array) $wp_settings_sections[ $page ] as $section ) {
        if ( $section['title'] ) {
            echo "<h2>{$section['title']}</h2>\n";
        }
 
        if ( $section['callback'] ) {
            call_user_func( $section['callback'], $section );
        }
 
        if ( ! isset( $wp_settings_fields ) || ! isset( $wp_settings_fields[ $page ] ) || ! isset( $wp_settings_fields[ $page ][ $section['id'] ] ) ) {
            continue;
        }

        //do_settings_fields( $page, $section['id'] );
		$sectionId= $section['id'];

		/* do_settings_fields_properly*/
		if ( ! isset( $wp_settings_fields[ $page ][ $sectionId ] ) ) {
			return;
		}
	 
		foreach ( (array) $wp_settings_fields[ $page ][ $sectionId ] as $field ) {
			call_user_func( $field['callback'], $field['args'] );
		}
		/* do_settings_fields_properly END*/
    }


}


add_action( 'admin_init', function(){

	$settingName= "ash-wp-settings";
	$sectionName = "ash-wp-config";
	
	$configFields = [
		"tagOptions" => ["label"=>"Tag Options", "default"=>true, "type"=>"boolean", 
			"description"=>"Adds the ability to customise the tag when you use you wp_include_style/script, you can add attributes use [] in the url e.g. \"myfile.js[async][type=module].\" "],
		"disableIncludes" => ["label"=>"Remove Included files", "default"=>\AshWP::Instance()->GetDefaultIncludes(), "type"=>"array", 
			"description"=>""],
		"disableHeadData" => ["label"=>"Cleanup Head Tag Clutter", "default"=>\AshWP::Instance()->GetDefaultHeadTags(), "type"=>"array", 
			"description"=>""],
		"disableFeatures" => ["label"=>"Disable Advanced BlogFeatures", "default"=>\AshWP::Instance()->GetDefaultFeatures(), "type"=>"array", 
			"description"=>""],
		"adminBar" => ["label"=>"Admin Bar", "default"=>true, "type"=>"boolean", 
			"description"=>"Allows you to hide the admin bar, mainly for debuggin UI and fixed elements that hide"],
		"pageData" => ["label"=>"Page Data", "default"=>false, "type"=>"boolean", 
			"description"=>"Dumps a bunch of debug information in an object and logs it to the terminal."],
	];



	register_setting( "ash-wp", $settingName, array("sanitize_callback"=>function($data)use($configFields){ return ParseInput($data, $configFields);}) );

	add_settings_section( $sectionName, "Config Section", null, "ash-wp" );
	

	$configKeys = array_keys($configFields);

	for($i=0;$i<count($configKeys);$i++)
	{
		$key = $configKeys[$i];
		$item = (object)$configFields[$key];
		$item->key = $key;
		$item->setting =$settingName;
		$item->section =$sectionName;
		
		switch($item->type)
		{
			case "bool":
			case "boolean":
				add_settings_field($key, $item->label, function($output) use($item){PrintTogleSwitch($item, $output);}, "ash-wp", "ash-wp-config"  );
				break;
			case "array":
				add_settings_field($key, $item->label, function($output) use($item){PrintTextArea($item, $output);}, "ash-wp", "ash-wp-config"  );
		}
	}

} );

function ParseInput($inputData, $fieldData)
{

	if($inputData == null)
		$inputData = array();

	$fieldKeys = array_keys($fieldData);
	$output = array();

	for($i=0; $i<count($fieldKeys); $i++)
	{
		$key = $fieldKeys[$i];
		$fieldInfo = (object)$fieldData[$key];

		$fieldInput = array_key_exists($key, $inputData) ? $inputData[$key]: null;

		if($fieldInput !== null)
		{	
			switch($fieldInfo->type)
			{
				case "bool":
				case "boolean":
					$fieldInput= json_decode($fieldInput); //prone to errors
					break;
				case "array":
					$fieldInput = TextAreaToArray($fieldInput);
					break;
				
			}
		}
		else
		{
			$fieldInput = $fieldInfo->default;
		}

		$output[$key] = $fieldInput;

	}
	
	return $output;
}


function PrintTogleSwitch($item, $option)
{
	$setting = get_option($item->setting, array());
	$setting = empty($setting) ? array() : $setting;
	$value = array_key_exists($item->key, $setting) ? $setting[$item->key] : $item->default ;

?>

<div>
	<h3><?=$item->label?></h3>
	<input type="hidden" name="<?=$item->setting?>[<?=$item->key?>]" value=false />
	<label class="switch">
		<input type="checkbox" value="true" name="<?=$item->setting?>[<?=$item->key?>]" <?=checked($value)?> />
		<span class="slider"></span>
	</label>
	<p><?=$item->description?></p>
</div>
<?php
}

function PrintTextArea($item, $option)
{

	$setting = get_option($item->setting, array());
	$setting = empty($setting) ? array() : $setting;
	$value = array_key_exists($item->key, $setting) ? $setting[$item->key] : $item->default ;

?>


<div class="list-area">
	<h3><?=$item->label?></h3>
	<textarea name="<?=$item->setting?>[<?=$item->key?>]"><?=implode("\n", $value)?></textarea>
</div>
<?php
}

function CheckboxToBool($value)
{
	return $value?1:0;
}

function TextAreaToArray($value)
{
	$newValue = $value;
	$newValue = sanitize_textarea_field($newValue);
	$newValue = explode("\n", $newValue);
	$newValue = array_map('trim', $newValue);


	return $newValue;
}